/****
 * SDL_EpocAudio.h
 * 
 * Copyright (C) 2003 Andreas Karlsson
 * All rights reserved
 */ 

#include <string.h> //memset

#include <MdaAudioOutputStream.h>
#include <mda\common\audio.h>


#include "SDL_EpocAudio.h"
#include "SDL_timer.h"

#include "PolledAs.h"

static int EPOC_OpenAudio(_THIS, SDL_AudioSpec *spec);
static void EPOC_WaitAudio(_THIS);
static void EPOC_ThreadInit(_THIS);
static Uint8 *EPOC_GetAudioBuf(_THIS);
static void EPOC_PlayAudio(_THIS);
static void EPOC_WaitDone(_THIS);
static void EPOC_CloseAudio(_THIS);

typedef void (*fill_soundbuffer_callback)(void *userdata, Uint8 *stream, int len);

SDL_AudioSpec gSpec;

class CStreamEngine : public MMdaAudioOutputStreamCallback, public CBase
{
public:

	enum TState
    {
        /** No operation */
		EIdle,
		/** Preparing to play a stream */
		EStreamPrepare,
		/** Stream initiated */
		EStreamStarted,
		/** Requires new stream data */
		ENextStreamBuf,
		/** Error in playing stream */
		EStreamError,
		/** Stream closing down */
		EStreamStopping
	};


	static CStreamEngine *NewL();

	~CStreamEngine();

	void OpenL(TInt aSampleRate, TInt aChannels);

	void Close();

	void StartStreamL();

	TUint8 *Buffer();

	TState State();

	void WaitForOpenToCompleteL();

protected:
	void ConstructL();

	void WriteL( /*TDesC8 &aBuffer */ );


// Implement MMdaAudioOutputStreamCallback
	void MaoscPlayComplete(TInt aError);

	void MaoscBufferCopied(TInt aError, const TDesC8& /*aBuffer*/);

	void MaoscOpenComplete( TInt aError );

	void StreamCleanup();

	CMdaAudioOutputStream		*iAudioStreamPlayer;
	TMdaAudioDataSettings		 iSettings;

	TUint8						 iVolume;

	TState						 iState;

	HBufC8*						iBuffer;
//	TPtr8						*iBufferPtr;

	TBool						 iIsOpen;
	TBool					  	 iIsStarted;

};



CStreamEngine *CStreamEngine::NewL()
{
	CStreamEngine *self = new(ELeave) CStreamEngine;
	CleanupStack::PushL( self );
	self->ConstructL();
	CleanupStack::Pop();

	return self;
}

CStreamEngine::~CStreamEngine()
{
	if( iAudioStreamPlayer )
		delete iAudioStreamPlayer;

	delete iBuffer;
}

void CStreamEngine::OpenL(TInt aSampleRate, TInt aChannels)
{
	iSettings.Query();
	iSettings.iSampleRate = aSampleRate;
	iSettings.iChannels = aChannels;
	iSettings.iFlags = 0;
	iSettings.iVolume = 0;

	TInt err = 0;
	TRAP(err, iAudioStreamPlayer->Open( &iSettings ));
	if(err)
		User::Panic(_L("SDL AUDIO"), 25);

	TRAP(err, WaitForOpenToCompleteL());
	if(err)
		User::Panic(_L("SDL AUDIO"), 23);
	iState = EStreamStarted;

	iSettings.iChannels		= aChannels;
	iSettings.iSampleRate	= aSampleRate;
	TRAP(err, iAudioStreamPlayer->SetAudioPropertiesL( iSettings.iSampleRate, iSettings.iChannels ));
	if(err)
		User::Panic(_L("SDL AUDIO"), err);

	iAudioStreamPlayer->SetVolume( iAudioStreamPlayer->MaxVolume() );

}

void CStreamEngine::Close()
{
	iAudioStreamPlayer->Stop();
}

void CStreamEngine::StartStreamL()
{

	WriteL( /**iBufferPtr */);
}

TUint8 *CStreamEngine::Buffer()
{
	if(iState == EStreamStarted)
		return (TUint8*)iBuffer->Des().Ptr();
	return 
		NULL;
}

CStreamEngine::TState CStreamEngine::State()
{
	return iState;
}



void CStreamEngine::WaitForOpenToCompleteL()
{
	iIsStarted=ETrue;

	CActiveScheduler::Start();
	if (!iIsOpen)
	{
		User::Panic(_L("SDL AUDIO"), 30);
	}
}

void CStreamEngine::ConstructL()
{
	iIsStarted=EFalse;
	iAudioStreamPlayer = CMdaAudioOutputStream::NewL( *this );
	iBuffer=HBufC8::NewL(gSpec.size);
	iBuffer->Des().SetLength(gSpec.size);

}

void CStreamEngine::WriteL( /*TDesC8 &aBuffer */ )
{
	iAudioStreamPlayer->WriteL( *iBuffer );
	iIsStarted=ETrue;
	CActiveScheduler::Start();
}


// Implement MMdaAudioOutputStreamCallback
void CStreamEngine::MaoscPlayComplete(TInt aError)
{
	User::Panic(_L("Play Complete"), aError );
	if(aError)
		StreamCleanup();
	else
		StreamCleanup();
	
	if(	iIsStarted)
	{
		CActiveScheduler::Stop();
		iIsStarted=EFalse;
	}
}

void CStreamEngine::MaoscBufferCopied(TInt aError, const TDesC8& /*aBuffer*/)
{

	// Check for error
	if (aError)
		iState = EStreamError;
	else
		iState = ENextStreamBuf;
	if(	iIsStarted)
	{
		CActiveScheduler::Stop();
		iIsStarted=EFalse;
	}

	//Tell the SDL client that it can put more buffers :D

}

void CStreamEngine::MaoscOpenComplete( TInt aError )
{

	if( aError )
	{
		User::Panic(_L("SDL_AUDIO_INIT"), aError);
		StreamCleanup();
	}
	else
	{
//		User::Panic(_L("SDL_AUDIO_INIT"), aError);
		iAudioStreamPlayer->SetVolume( iAudioStreamPlayer->MaxVolume() /*/ iVolume*/ );

		iState = EStreamStarted;

		iIsOpen = ETrue;
	}
	
	if(	iIsStarted)
	{
		CActiveScheduler::Stop();
		iIsStarted=EFalse;
	}}


void CStreamEngine::StreamCleanup()
{
	//Delete the iAudioStreamPlayer or what?
	iState = EIdle;
}





static int Audio_Available()
{
	return 1;
}
	
static CActiveScheduler*	sPolledActiveScheduler = NULL;

static void Audio_DeleteDevice( _THIS )
{
//	_this->hidden.iStreamEngine.Stop(); //necessary?
	delete _this->hidden->iStreamEngine;
	delete _this->hidden->iScheduler;
	delete _this->hidden->iCleanup;
	delete _this->hidden;
	delete _this;

	delete sPolledActiveScheduler;
}

static SDL_AudioDevice *Audio_CreateDevice( int devindex )
{

	_THIS;

	_this = new(ELeave) SDL_AudioDevice;

	memset(_this, 0, sizeof(SDL_AudioDevice));
	_this->hidden = new(ELeave) SDL_PrivateAudioData;
	memset(_this->hidden, 0, sizeof(SDL_PrivateAudioData));

	_this->hidden->iStreamEngine	= NULL; //Has to be allocated in player thread... 
	_this->hidden->iCleanup			= NULL; //Has to be allocated in player thread...  ??

	_this->OpenAudio	= EPOC_OpenAudio;
	_this->ThreadInit	= EPOC_ThreadInit;
	_this->WaitAudio	= EPOC_WaitAudio;
	_this->PlayAudio	= EPOC_PlayAudio;
	_this->GetAudioBuf	= EPOC_GetAudioBuf;
	_this->WaitDone		= EPOC_WaitDone;
	_this->CloseAudio	= EPOC_CloseAudio;
	_this->free			= Audio_DeleteDevice;
	return _this;
}

AudioBootStrap	EPOC_audiobootstrap = 
{
	  "EPOC_media_stream"
	, "Symbian 7.0 Audio Streaming"
	, Audio_Available
	, Audio_CreateDevice
};

#include <e32svr.h>
void EPOC_ThreadInit( _THIS )
{
	_this->hidden->iCleanup = CTrapCleanup::New();
	RDebug::Print(_L("Init SDL audio thread"));

	CActiveScheduler::Install(NULL); //Uninstall old scheduler
//	_this->hidden->iScheduler = new (ELeave) CActiveScheduler;
//	CActiveScheduler::Install(_this->hidden->iScheduler);

	//Install Active Scheduler
	TRAPD(err, sPolledActiveScheduler = new(ELeave)CActiveScheduler);//CPolledActiveScheduler::NewL()) 
	if(err)
		User::Panic(_L("SDL Audio"), err);
	CActiveScheduler::Install(sPolledActiveScheduler); 


	TRAP(err, _this->hidden->iStreamEngine = CStreamEngine::NewL());
	if(err)
		User::Panic(_L("SDL Audio"), err);

	TInt channels=0;
	TInt sampleRate=0;

	switch( gSpec.channels )
	{
	case 1:	channels = TMdaAudioDataSettings::EChannelsMono; break;
	case 2: channels = TMdaAudioDataSettings::EChannelsStereo; break;
	default:
		User::Panic(_L("SDL AUDIO"), -2);
	}

	switch( gSpec.format & 0xFF )
	{
	case 8:
		User::Panic(_L("SDL_AUDIO"), -1 );
		break;
	case 16:
		gSpec.format = AUDIO_S16;
		
		switch( gSpec.freq )
		{
		case  8000: sampleRate = TMdaAudioDataSettings::ESampleRate8000Hz;  break;
		case 11025: sampleRate = TMdaAudioDataSettings::ESampleRate11025Hz; break;
		case 22050: sampleRate = TMdaAudioDataSettings::ESampleRate22050Hz; break;
		case 32000: sampleRate = TMdaAudioDataSettings::ESampleRate32000Hz; break;
		case 44100: sampleRate = TMdaAudioDataSettings::ESampleRate44100Hz; break;
		case 48000: sampleRate = TMdaAudioDataSettings::ESampleRate48000Hz; break;
		}
	}
	
//	int err=0;

	TRAP(err, _this->hidden->iStreamEngine->OpenL( sampleRate, channels ));
	gSpec.channels=2;
	if(err)
		User::Panic(_L("SDL Audio"), err);
	
}

int EPOC_OpenAudio(_THIS, SDL_AudioSpec *aSpec)
{
	//Needs to open in the same thread as playing is going to be :|
	gSpec = *aSpec;

//	_this->hidden->iStreamEngine->WaitForOpenToCompleteL();

	return 0;
}

void EPOC_CloseAudio( _THIS )
{
	_this->hidden->iStreamEngine->Close();
}

void EPOC_WaitDone( _THIS )
{
	//While the stream engine is working loop and wait :)
	//while( _this->hidden->iStreamEngine->State() != CStreamEngine::EIdle )
	//	SDL_Delay(100);
}

void EPOC_PlayAudio( _THIS )
{

	_this->hidden->iStreamEngine->StartStreamL();
	//if (sPolledActiveScheduler)
	//	sPolledActiveScheduler->Schedule();

}

Uint8 *EPOC_GetAudioBuf(_THIS)
{
	return _this->hidden->iStreamEngine->Buffer();
}

void EPOC_WaitAudio( _THIS )
{
	//_this->hidden->iStreamEngine->WaitForCopyToCompleteL(); //This is the best we can do right? 
}

