/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Sam Lantinga
    slouken@devolution.com
*/

/*
    SDL_main.cpp
    The Epoc executable startup functions 

    Epoc version by Hannu Viitala (hannu.j.viitala@mbnet.fi)
*/

#include <e32std.h>
#include <e32def.h>
#include <e32svr.h>
#include <e32base.h>
#include <estlib.h>
#include <stdlib.h>
#include <stdio.h>
#include <w32std.h>
#include <apgtask.h>

#include "SDL_error.h"

#if defined(__WINS__)
#include <estw32.h>
IMPORT_C void RegisterWsExe(const TDesC &aName);
#endif

/* The prototype for the application's main() function */
#define main	SDL_main
extern "C" int main (int argc, char *argv[], char *envp[]);
extern "C" void exit (int ret);



//CActiveScheduler *gEPOCActiveScheduler;
/* Epoc main function */

GLDEF_C TInt E32Main() 
    {

	_LIT(KHeapName, "SDL HEAP" );

	RHeap *heap = UserHeap::ChunkHeap(&KHeapName, 0x100, 0xFFFFFF, 0xFA00 ); //Max 16M, grow by 64K
	RHeap *oldheap = User::SwitchHeap( heap );

    /*  Get the clean-up stack */
	CTrapCleanup* cleanup = CTrapCleanup::New();
//	CActiveScheduler *gEPOCActiveScheduler = new CActiveScheduler;

//	CActiveScheduler::Install( gEPOCActiveScheduler );
//	scheduler->Start(); Has to be in an own thread??
    #if defined(__WINS__)
    /* arrange for access to Win32 stdin/stdout/stderr */
   	RWin32Stream::StartServer();	
    #endif

    /* Arrange for multi-threaded operation */
	SpawnPosixServerThread();	

    /* Get args and environment */
	int argc=0;
	char** argv=0;
	char** envp=0;
	__crt0(argc,argv,envp);	

    #if defined(__WINS__)
	// Cause the graphical Window Server to come into existence 
	RSemaphore sem;
	sem.CreateGlobal(_L("WsExeSem"),0);
	RegisterWsExe(sem.FullName());
    #endif


	//Test...
/*	RTextInputServerClient  textInputClient;
	User::LeaveIfError( textInputClient.Connect() );
	User::LeaveIfError( textInputClient.ActivateWriteAnywhereTi() );
*/

    /* Start the application! */

    /* Create stdlib */
    _REENT;

    /* Set process and thread priority */
    RThread currentThread;

    currentThread.Rename(_L("SdlProgram"));
    currentThread.SetProcessPriority(EPriorityLow);
    currentThread.SetPriority(EPriorityMuchLess);

    /* Call stdlib main */
    int ret=0;
	
 /* !! process exits here if there is "exit()" in main! */	
	TRAPD(err, ret = main(argc, argv, envp));
	if( err != KErrNone )
		User::Panic(_L("SDLMain"), err);

    /* Call exit */
    exit(ret); /* !! process exits here! */			

    /* Free resources and return */
    CloseSTDLIB();
   	delete cleanup;	
//	delete 	gEPOCActiveScheduler;
//	textInputClient.Close();

	User::SwitchHeap( oldheap ); //Do not know if I need this...

    return(KErrNone);
    }

/* Epoc dll entry point */
#if defined(__WINS__)
GLDEF_C TInt E32Dll(TDllReason)
	{
	return(KErrNone);
	}

EXPORT_C TInt WinsMain(TAny *)
	{
    E32Main();
	return KErrNone;
	}
#endif


