/*******************************************************************
 * SDLRunner.cpp
 *
 * Author:	Andreas Karlsson (sprawl@onlajn.nu)
 * (c) Copyright 2002, Andreas Karlsson
 * All Rights Reserved
 *
 */

#include <EikEnv.h>
#include "SDLRunner.h"

const TInt KSDLStackSize	= 0x10000;			// 64KB
const TInt KSDLHeapSizeMin	= 0x10000;			// 64KB
const TInt KSDLHeapSizeMax	= 0x2000000;		// 32 MB


extern "C" int main(int argc, char *argv[]);

//////
// CSDLRunner implementation
// public:
CSDLRunner *CSDLRunner::NewL()
{
	CSDLRunner*	self = new(ELeave) CSDLRunner;
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop();			// self
	return self;
}

CSDLRunner::~CSDLRunner()
{
	Cancel();
}


void CSDLRunner::RunSDLL()
{
	User::LeaveIfError( iChild.Create(_L("SDL THREAD"), CSDLRunner::ThreadMain, KSDLStackSize, KSDLHeapSizeMin, KSDLHeapSizeMax, this, EOwnerThread ) );

	iChild.Logon( iStatus );
	SetActive();

	iChild.Resume();
}

void CSDLRunner::TerminateSDLL()
{
	
	Cancel();

	if( iChild.Handle() )
	{
		iChild.Terminate(0); //Reason 0
		iChild.Close();
	}
}

//Protected
CSDLRunner::CSDLRunner()
	: CActive(CActive::EPriorityStandard)
{
	
}

void CSDLRunner::ConstructL()
{
	CActiveScheduler::Add( this );
}

//Implement CActive
void CSDLRunner::RunL()
{
	//Thread died
}

void CSDLRunner::DoCancel()
{
	iChild.LogonCancel( iStatus );

}


TInt CSDLRunner::ThreadMain(TAny* aData)
{
	CTrapCleanup* cleanup=CTrapCleanup::New();

	CSDLRunner*	self = (CSDLRunner*) aData;
	TRAPD(err, self->DoRunGameL());

	delete cleanup;

	return(err);	
}

void CSDLRunner::DoRunGameL()
{
	char *argv[] = {{"esdl"}};
	main( 0, NULL );
}
