#ifndef __OIM_QIK_QOIMLISTVIEW_H__
#define __OIM_QIK_QOIMLISTVIEW_H__

#include <coecntrl.h>
#include <coeview.h>
#include <eiklbo.h>

#include "QSDLViewBase.h"
#include "QSDLModel.h"

class CEikListBox;
class CFont;

// A base class to encapsulate the common features of both the text and
// graphics list views. It holds an array of the shapes (filtered by the 
// choice of category) that should be displayed, and handles switching to 
// the detailed view when the user clicks on an item in a view's list box
class CQSDLListView : public CQSDLViewBase, public MEikListBoxObserver
	{
public:
	// Array of pointers to objects in current list view 
	typedef RPointerArray<void *> TCSPtrArray;

public:
	~CQSDLListView();

	// Gets array of objects in current list view (i.e. all in category)
	const TCSPtrArray& ObjectsInView() const;

	// Implements CQSDLViewBase
	void HandleCategoryChangeL(TInt aHandle);

	// Implements MCoeView
	void ViewActivatedL(const TVwsViewId& aPrevViewId,TUid aCustomMessageId,const TDesC8& aCustomMessage);
	void ViewDeactivated();

	// Implements MEikListBoxObserver
	void HandleListBoxEventL(CEikListBox* aListBox, TListBoxEvent aEventType);
	
protected:
	CQSDLListView();
	// Set the list box for the view
	void SetListBox(CEikListBox& aListBox);
	// List views should implement this to update the list box 
	// in response to category changes
	virtual void RefreshListBoxL() = 0;

protected:
	// Array of shapes in the view
	// (would be better if this were static)
	TCSPtrArray iObjectsInView;

private:
	// Refilters shape array for a category
	void SetListObjects(TInt aHandle);

private:
	CEikListBox* iListBox;
	};


#endif //__OIM_QIK_QOIMLISTVIEW_H__