#include <uikon.hrh>
#include <oIM.rsg>
#include <eikmenup.h>
#include <eikmenub.h>
#include <qikon.hrh>
#include <eikenv.h>
#include <qikcategorymodel.h>
#include <qikdocument.h>

#include "QSDLUi.h"
#include "QSDLModel.h"
//#include "CQSDLGraphicsListView.h"
#include "QSDLUserListView.h"
#include "QSDLDocument.h"
#include "oIM.hrh"
#include "q_sdl_external_interface.h"


CQSDLUi::CQSDLUi()
:iCurrentCategory(EAll)
	{
	}


// Creates the views, registers them and sets the graphics list view as the default
void CQSDLUi::ConstructL()
{
	CQikAppUi::ConstructL();

	// Initialise category model
	iCategoryModel = QikCategoryUtils::ConstructCategoriesLC(R_OIM_DEFAULT_CATEGORIES);
	CleanupStack::Pop(); // iCategoryModel
	
	// Set the folder menu to category "all" at startup
	iEikonEnv->AppUiFactory()->MenuBar()->SetTitleL(iCategoryModel->
		CategoryNameByHandle(EAll), R_OIM_LIST_CAT_MENU);

	/*
	iGraphicsListView = CQSDLGraphicsListView::NewL();
	RegisterViewL(*iGraphicsListView);
	AddToStackL(*iGraphicsListView, iGraphicsListView);
	*/

	iUserListView = CQSDLUserListView::NewL();
	RegisterViewL(*iUserListView);
	AddToStackL(*iUserListView, iUserListView);
			    

	SetDefaultViewL(*iUserListView);
}

// Gets a pointer to the current view
CQSDLViewBase* CQSDLUi::CurrentView()
	{
	CQSDLViewBase* curView = NULL;
	TVwsViewId id;

	TInt err = GetActiveViewId(id);
	if(err == KErrNone)
		{
		if(id == iUserListView->ViewId())
			curView = iUserListView;
		}
	return curView;
	}

// On destruction, deregister and remove all views
CQSDLUi::~CQSDLUi()
	{
	if(iUserListView) // safety precaution
		{
		RemoveFromStack(iUserListView);
		DeregisterView(*iUserListView);
		}
	delete iUserListView;
	delete iCategoryModel;
	}

// Handle command input for all views
void CQSDLUi::HandleCommandL(TInt aCommand)
	{
	switch (aCommand)
		{
		// Handle graphics view button, and details view's go back button
		case EReturnToListView:
//		case ESwitchToGraphicsView:
            {
			SwitchViewL(KUidQSDLUserListView,*iUserListView);
			break;
			}
		// Handles exit command
        case EEikCmdExit:
			{
			Exit();
			break;
			}
		case EConnectToServer:
			{
				break;
			}

		}
	}

// Helper function to switch views
void CQSDLUi::SwitchViewL(const TUid& aTargetViewId, 
							  CQSDLViewBase& aTargetView)
	{
	if (CurrentView() != &aTargetView)
		{
		TVwsViewId id(KUidSDL, aTargetViewId);
		ActivateViewL(id);
		}
	}

// Pass category selections onto list views
void CQSDLUi::HandleCategoryChangeL(TInt aHandle)
	{
	iCurrentCategory = aHandle;
	iUserListView->HandleCategoryChangeL(aHandle);
	}

// Initialise menus
void CQSDLUi::DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane)
	{
	switch(aResourceId)
		{
//#ifdef __WINS__
//#ifdef _DEBUG
// This gives us the option to close down our application, letting the framework
// check for memory and resource leaks. This menu option should not exist in
// released versions, thus the ifdef
		case R_OIM_LIST_OIM_MENU:
		case R_OIM_DETAILED_OIM_MENU:
			{
			CEikMenuPaneItem::SData itemData;

			itemData.iText=_L("Connect");
			itemData.iCommandId=EConnectToServer;
			itemData.iFlags=0;
			itemData.iCascadeId=0;
			aMenuPane->AddMenuItemL( itemData );

			itemData.iText=_L("Close");
			itemData.iCommandId=EEikCmdExit;
			itemData.iFlags=0;
			itemData.iCascadeId=0;
			aMenuPane->AddMenuItemL(itemData);
			}
			break;
//#endif
//#endif
		case R_OIM_LIST_CAT_MENU:
			DynInitCategoryMenuPaneL(aMenuPane);
			break;
		}
	}

void CQSDLUi::DynInitMenuBarL(TInt aResourceId, CEikMenuBar* aMenuBar)
	{
	if ((aResourceId == R_OIM_LIST_MENUBAR) && iCategoryModel)
		aMenuBar->SetTitleL(iCategoryModel->
			CategoryNameByHandle(CurrentCategory()), R_OIM_LIST_CAT_MENU);
	}

TInt CQSDLUi::CurrentCategory()
	{
	return iCurrentCategory;
	}
