#include <coemain.h>
#include "QSDLDirectDisplay.h"


CQSDLDirectDisplay *CQSDLDirectDisplay::NewL( RWsSession& aClient, RWindow& aWindow, CWsScreenDevice &aDevice  )
{
	CQSDLDirectDisplay *self = NewLC( aClient, aWindow, aDevice );
	CleanupStack::Pop();
	return self;
}

CQSDLDirectDisplay *CQSDLDirectDisplay::NewLC( RWsSession& aClient, RWindow& aWindow, CWsScreenDevice &aDevice  )
{
	CQSDLDirectDisplay *self = new(ELeave) CQSDLDirectDisplay( aClient, aWindow );

	CleanupStack::PushL( self );
	self->ConstructL( aDevice );
	return self;
}

CQSDLDirectDisplay::~CQSDLDirectDisplay()
{
	Cancel();
	delete iDirectScreenAccess;
}

void CQSDLDirectDisplay::StartL()
{
	iDirectScreenAccess->StartL();
	iGc = iDirectScreenAccess->Gc();
	
	iRegion = iDirectScreenAccess->DrawingRegion();

	iGc->SetClippingRegion( iRegion );
}

void CQSDLDirectDisplay::Cancel()
{
	iDirectScreenAccess->Cancel();
}

CFbsBitGc	&CQSDLDirectDisplay::Gc()
{
	return *iGc;
}

void CQSDLDirectDisplay::Update()
{

/**
 * From SDK:
 *   Code built to run on the Emulator must call CFbsScreenDevice::Update() in order to see the results of drawing to this graphics context, but this may not be required on all target hardware. 
 *
 * I Really dislike this kind of special treat :|
 */

#ifdef __WINS__
//	iDirectScreenAccess->ScreenDevice()->Update();
#endif 
}

void CQSDLDirectDisplay::Flush()
{
	iClient.Flush();
}

void CQSDLDirectDisplay::Restart(RDirectScreenAccess::TTerminationReasons)
{
	//Not a leave method, should we trap this? 
	StartL();
}

void CQSDLDirectDisplay::AbortNow(RDirectScreenAccess::TTerminationReasons)
{
	Cancel();
}

void CQSDLDirectDisplay::ConstructL( CWsScreenDevice &aDevice )
{
	iDirectScreenAccess = CDirectScreenAccess::NewL(
		iClient,									// WS session
		aDevice ,		// CWsScreenDevice
		iWindow,									// RWindowBase
		*this										// MDirectScreenAccess
		);

}

CQSDLDirectDisplay::CQSDLDirectDisplay( RWsSession& aClient, RWindow& aWindow )
	: iClient( aClient )
	, iWindow( aWindow )
{
}