/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Sam Lantinga
    slouken@devolution.com
*/

/*
    SDL_epocevents.cpp
    Handle the event stream, converting Epoc events into SDL events

    Epoc version by Hannu Viitala (hannu.j.viitala@mbnet.fi)
*/


#include <stdio.h>
#undef NULL

extern "C" {
#include "SDL_error.h"
#include "SDL_video.h"
#include "SDL_keysym.h"
#include "SDL_keyboard.h"
#include "SDL_events_c.h"
#include "SDL_timer.h"
}; /* extern "C" */

#include "SDL_epocvideo.h"
#include "SDL_epocevents_c.h"

#include <hal.h>

#ifndef SYMBIAN_SERIES_60
#include <QuartzKeys.h>
#endif

extern "C" {
/* The translation tables from a console scancode to a SDL keysym */
static SDLKey keymap[MAX_SCANCODE];
static SDL_keysym *TranslateKey(int scancode, SDL_keysym *keysym);
}; /* extern "C" */

TBool isCursorVisible = ETrue;
TInt iLastChar=-1;
TInt iLetterOffset=0;
extern bool gViewVisible;

bool gRightClickSet=false;
TBool IsScreenKeysL(const TPointerEvent& aPointerEvent)
{
#ifndef SYMBIAN_SERIES_60
	switch(aPointerEvent.iType)
				{
				case TPointerEvent::EButton1Down:
					{
						if(aPointerEvent.iPosition.iX>199)
						{
							TRect rect;
							for(TInt loop=0;loop<20;loop++)
							{
								rect=TRect(TPoint(200,loop*16),TSize(16,16));
								if(rect.Contains(aPointerEvent.iPosition))
								{
									switch(loop)
									{
									case 0: // enter
										iLastChar=EStdKeyEnter;
										break;
									case 1: //rightclick set
										gRightClickSet=!gRightClickSet;
										// need to repaint lower bottom
										static_cast<CEBasicAppUi*>(CEikonEnv::Static()->EikAppUi())->DrawView();
										break;
									case 2: // prev
										if(iLetterOffset>0)
										{
											iLetterOffset-=8;
											if(iLetterOffset<0)
												iLetterOffset=0;
											static_cast<CEBasicAppUi*>(CEikonEnv::Static()->EikAppUi())->View()->UpdateCharSelector(iLetterOffset);
										}
										break;
									case 19:// next
										if(iLetterOffset<KOnScreenChars().Length()-1)
										{
											iLetterOffset+=8;
											if(iLetterOffset>KOnScreenChars().Length()-1)
												iLetterOffset=KOnScreenChars().Length()-1;

											static_cast<CEBasicAppUi*>(CEikonEnv::Static()->EikAppUi())->View()->UpdateCharSelector(iLetterOffset);
										}
										break;
									default: // chars in array
										{
											TInt charIndex=(loop-3)+iLetterOffset;
											if(charIndex<KOnScreenChars().Length())
											{
												iLastChar=KOnScreenChars()[charIndex];
											}
										}
										break;
									}
									break;
								}
							}
							if(iLastChar!=-1)
							{
								TInt scancode = iLastChar;
								SDL_keysym keysym;
								TChar key(scancode);
								if(key.IsLower())
								{
									scancode=key.GetUpperCase();
								}else if(key.IsUpper())
								{
									SDL_PrivateKeyboard(SDL_PRESSED, TranslateKey( EStdKeyLeftShift, &keysym));
								}
								
								(void*)TranslateKey(scancode, &keysym);							
								SDL_PrivateKeyboard(SDL_PRESSED, &keysym);
								if(key.IsUpper())
								{
									SDL_PrivateKeyboard(SDL_RELEASED, TranslateKey( EStdKeyLeftShift, &keysym));
									
								}
								return ETrue;
							}
						}
					}
					break;
				case TPointerEvent::EButton1Up:
					{
						if(iLastChar!=-1)
						{
							TInt scancode = iLastChar;
							SDL_keysym keysym;
							TChar key(scancode);
							if(key.IsLower())
							{
								scancode=key.GetUpperCase();
							}
							else if(key.IsUpper())
							{
								SDL_PrivateKeyboard(SDL_PRESSED, TranslateKey( EStdKeyLeftShift, &keysym));
							}
							
							(void*)TranslateKey(scancode, &keysym);							
							SDL_PrivateKeyboard(SDL_RELEASED, &keysym);
							
							if(key.IsUpper())
							{
								SDL_PrivateKeyboard(SDL_RELEASED, TranslateKey( EStdKeyLeftShift, &keysym));
								
							}
							iLastChar=-1;
							return ETrue;
						}
					}
					break;
				case TPointerEvent::EDrag:
					{
						if(iLastChar!=-1)
						{
							return ETrue;
					
						}
					}
					break;
				case TPointerEvent::EButton2Down:
				case TPointerEvent::EButton3Down:
				case TPointerEvent::EButton3Up:
					break;
					
				}
#endif // in S60 will always return false
	return EFalse;
}

// calculate the emulated mouse movement according to
// the screen rotation mode
//
// results are returned on the private SDL structure

void EPOC_SetEmulMouseMovement(_THIS, TUint32 aJoyKey )
{
	EPOC_RotateModes rotateMode = Private->EPOC_RotateMode;
	TInt mouseX = Private->EPOC_EmulMouseX;
	TInt mouseY = Private->EPOC_EmulMouseY;

	// check key pressed and rotation mode to change the mouse position
	// as the user would expect

	switch (aJoyKey)
	{
	case EStdKeyRightArrow:
		if(rotateMode == EPOC_ROTATE_NONE) mouseX++;
		if(rotateMode == EPOC_ROTATE_CLOCKWISE) mouseY--;
		if(rotateMode == EPOC_ROTATE_COUNTERCLOCKWISE) mouseY++;
		break;

	case EStdKeyLeftArrow:
		if(rotateMode == EPOC_ROTATE_NONE) mouseX--;
		if(rotateMode == EPOC_ROTATE_CLOCKWISE) mouseY++;
		if(rotateMode == EPOC_ROTATE_COUNTERCLOCKWISE) mouseY--;
		break;

	case EStdKeyUpArrow:
		if(rotateMode == EPOC_ROTATE_NONE) mouseY--;
		if(rotateMode == EPOC_ROTATE_CLOCKWISE) mouseX--;
		if(rotateMode == EPOC_ROTATE_COUNTERCLOCKWISE) mouseX++;
		break;

	case EStdKeyDownArrow:
		if(rotateMode == EPOC_ROTATE_NONE) mouseY++;
		if(rotateMode == EPOC_ROTATE_CLOCKWISE) mouseX++;
		if(rotateMode == EPOC_ROTATE_COUNTERCLOCKWISE) mouseX--;
		break;

	default:
		break;
	}

	// careful not to get out of the screen's coordinates
	if(mouseX < 0) mouseX = 0;
	if(mouseX > _this->screen->w) mouseX = _this->screen->w;
	if(mouseY < 0) mouseY = 0;
	if(mouseY > _this->screen->h) mouseY = _this->screen->h;

	// set the new values
	Private->EPOC_EmulMouseX = mouseX;
	Private->EPOC_EmulMouseY = mouseY;
}

int EPOC_HandleWsEvent(_THIS,const TWsEvent& aWsEvent)
{
    int posted = 0;
    SDL_keysym keysym;

	int scancode=0;

    switch (aWsEvent.Type()) {
    
    case EEventPointer: /* Mouse pointer events */
    {
        const TPointerEvent* pointerEvent = aWsEvent.Pointer();		
		if(!IsScreenKeysL(*pointerEvent))
		{
			TPoint mousePos = pointerEvent->iPosition;

#ifndef SYMBIAN_SERIES_60
			//SDL_TRACE1("SDL: EPOC_HandleWsEvent, pointerEvent->iType=%d", pointerEvent->iType); //!!

			if (Private->EPOC_ShrinkedHeight) {
				mousePos.iY <<= 1; /* Scale y coordinate to shrinked screen height */
			}
			
			if(_this->screen->h== 480 || _this->screen->h==240)
			{
				mousePos.iX=(mousePos.iX*1.2);
			}
			
			TInt scrWidth=320;
			if (Private->EPOC_ShrinkedWidth) {
				scrWidth=640;
				mousePos.iX <<= 1; /* Scale y coordinate to shrinked screen height */
			}
#endif

			//AK flip mouse x and y on UIQ
			posted += SDL_PrivateMouseMotion(0, 0, mousePos.iX, mousePos.iY); /* Absolute position on screen */
			//		posted += SDL_PrivateMouseMotion(0, 0, (-mousePos.iY)+scrWidth, mousePos.iX); /* Absolute position on screen */
			if (pointerEvent->iType==TPointerEvent::EButton1Down) 
			{
				
				posted += SDL_PrivateMouseButton(SDL_PRESSED, gRightClickSet?SDL_BUTTON_RIGHT:SDL_BUTTON_LEFT, 0, 0);
			}
			else if (pointerEvent->iType==TPointerEvent::EButton1Up) 
			{
				posted += SDL_PrivateMouseButton(SDL_RELEASED, gRightClickSet?SDL_BUTTON_RIGHT:SDL_BUTTON_LEFT, 0, 0);
			}
			else if (pointerEvent->iType==TPointerEvent::EButton2Down)
			{
				posted += SDL_PrivateMouseButton(SDL_PRESSED, SDL_BUTTON_RIGHT, 0, 0);
			}
			else if (pointerEvent->iType==TPointerEvent::EButton2Up) 
			{
				posted += SDL_PrivateMouseButton(SDL_RELEASED, SDL_BUTTON_RIGHT, 0, 0);
			}
			//!!posted += SDL_PrivateKeyboard(SDL_PRESSED, TranslateKey(aWsEvent.Key()->iScanCode, &keysym));
		}
        break;
    }
    
    case EEventKeyDown: /* Key events */
    {

		scancode = aWsEvent.Key()->iScanCode;

#ifdef SYMBIAN_SERIES_60
		if (scancode == EStdKeyDevice3)
		{
			posted += SDL_PrivateMouseMotion(0, 0,
				Private->EPOC_EmulMouseX, Private->EPOC_EmulMouseY);

			posted += SDL_PrivateMouseButton(SDL_PRESSED, SDL_BUTTON_LEFT, 0, 0);
			Private->EPOC_EmulButtonPressed = scancode;
			break;
		}

		if ( (scancode == EStdKeyRightArrow) || (scancode == EStdKeyLeftArrow)
			|| (scancode == EStdKeyUpArrow) || (scancode == EStdKeyDownArrow) )
		{
			EPOC_SetEmulMouseMovement(_this, scancode);
			posted += SDL_PrivateMouseMotion(0, 0, Private->EPOC_EmulMouseX, Private->EPOC_EmulMouseY); /* Absolute position on screen */
			Private->EPOC_EmulButtonPressed = scancode;
			break;
		}	
#endif

       (void*)TranslateKey(scancode, &keysym);
        
        /* Special handling */
        switch((int)keysym.sym) {
        case SDLK_CAPSLOCK:
            if (!isCursorVisible) {
                /* Enable virtual cursor */
	            HAL::Set(HAL::EMouseState, HAL::EMouseState_Visible);
            }
            else {
                /* Disable virtual cursor */
                HAL::Set(HAL::EMouseState, HAL::EMouseState_Invisible);
            }
            isCursorVisible = !isCursorVisible;
            break;
        }
        
	    posted += SDL_PrivateKeyboard(SDL_PRESSED, &keysym);
        break;
	} 

    case EEventKeyUp: /* Key events */
    {
		scancode = aWsEvent.Key()->iScanCode;

#ifdef SYMBIAN_SERIES_60
		if(scancode == EStdKeyDevice3)
		{
			posted += SDL_PrivateMouseMotion(0, 0, Private->EPOC_EmulMouseX, Private->EPOC_EmulMouseY);
			posted += SDL_PrivateMouseButton(SDL_RELEASED, SDL_BUTTON_LEFT, 0, 0);
			break;
		}

		if ( (scancode == EStdKeyRightArrow) || (scancode == EStdKeyLeftArrow)
			|| (scancode == EStdKeyUpArrow) || (scancode == EStdKeyDownArrow) )
		{
			Private->EPOC_EmulButtonPressed = 0;
			break;
		}

#endif

	    posted += SDL_PrivateKeyboard(SDL_RELEASED, TranslateKey( scancode, &keysym));
        break;

	} 
    
    case EEventFocusGained: /* SDL window got focus */
    {
        //Private->EPOC_IsWindowFocused = ETrue;
        /* Draw window background and screen buffer */
		gViewVisible=true;

        RedrawWindowL(_this);  
        break;
    }

    case EEventFocusLost: /* SDL window lost focus */
    {
        //Private->EPOC_IsWindowFocused = EFalse;

        // Wait and eat events until focus is gained again
        /*
	    while (ETrue) {
            Private->EPOC_WsSession.EventReady(&Private->EPOC_WsEventStatus);
            User::WaitForRequest(Private->EPOC_WsEventStatus);
		    Private->EPOC_WsSession.GetEvent(Private->EPOC_WsEvent);
            TInt eventType = Private->EPOC_WsEvent.Type();
		    Private->EPOC_WsEventStatus = KRequestPending;
		    //Private->EPOC_WsSession.EventReady(&Private->EPOC_WsEventStatus);
            if (eventType == EEventFocusGained) {
                RedrawWindowL(_this);
                break;
            }
	    }
        */
        break;
    }

    case EEventModifiersChanged: 
    {
	    TModifiersChangedEvent* modEvent = aWsEvent.ModifiersChanged();
        TUint modstate = KMOD_NONE;
        if (modEvent->iModifiers == EModifierLeftShift)
            modstate |= KMOD_LSHIFT;
        if (modEvent->iModifiers == EModifierRightShift)
            modstate |= KMOD_RSHIFT;
        if (modEvent->iModifiers == EModifierLeftCtrl)
            modstate |= KMOD_LCTRL;
        if (modEvent->iModifiers == EModifierRightCtrl)
            modstate |= KMOD_RCTRL;
        if (modEvent->iModifiers == EModifierLeftAlt)
            modstate |= KMOD_LALT;
        if (modEvent->iModifiers == EModifierRightAlt)
            modstate |= KMOD_RALT;
        if (modEvent->iModifiers == EModifierLeftFunc)
            modstate |= KMOD_LMETA;
        if (modEvent->iModifiers == EModifierRightFunc)
            modstate |= KMOD_RMETA;
        if (modEvent->iModifiers == EModifierCapsLock)
            modstate |= KMOD_CAPS;
        SDL_SetModState(STATIC_CAST(SDLMod,(modstate | KMOD_LSHIFT)));
        break;
    }
    default:            
        break;
	} 
	
    return posted;
}

SDL_VideoDevice * _thisDevice;

class CWaitForActivate:public CActive
{
public:
	CWaitForActivate();
	~CWaitForActivate();
	void RunL();
	void DoCancel();
	RTimer iTimer;
};

CWaitForActivate* gWaitForActive = NULL;

CWaitForActivate::CWaitForActivate():CActive(EPriorityStandard)
{
	CActiveScheduler::Add(this);
	iTimer.CreateLocal();
	iStatus=KRequestPending;
	iTimer.After(iStatus,250000);
	SetActive();
}

CWaitForActivate::~CWaitForActivate()
{
	iTimer.Close();
}


void CWaitForActivate::RunL()
{
	if(gViewVisible)
	{
        RedrawWindowL(_thisDevice);  
		CActiveScheduler::Stop(); 
	}
	else
	{
		iStatus=KRequestPending;
		iTimer.After(iStatus,250000);
		SetActive();
	}
}

void CWaitForActivate::DoCancel()
{
	iTimer.Cancel();
}

static TInt StopSchedulerTimeOut(TAny* /*any*/)
{
	TRawEvent	event;
	event.Set(TRawEvent::EActive);// keep us happy even if no events has been posted
	UserSvr::AddEvent(event); 
	if(!gViewVisible)
	{
		delete gWaitForActive;
		gWaitForActive = NULL;
		gWaitForActive = new CWaitForActivate;
	}
	else
		CActiveScheduler::Stop(); 
	return 0;
}

extern "C" {


void EPOC_PumpEvents(_THIS)
{
    int posted = 0; // !! Do we need this?
	_thisDevice=_this;
#ifdef SYMBIAN_SERIES_60

	// in series 60 we don't get repeated calls for a joystick key being pressed.
	// so while the key is down we continue moving the mouse. this piece of code
	// must be moved from here because the whole program gets stuck while there's
	// mouse movement.

	// a real solution would be to use a timer active object for the mouse

	TUint scancode = Private->EPOC_EmulButtonPressed;

	if ( (scancode == EStdKeyRightArrow) || (scancode == EStdKeyLeftArrow)
		|| (scancode == EStdKeyUpArrow) || (scancode == EStdKeyDownArrow) )
	{
		EPOC_SetEmulMouseMovement(_this, scancode);
		posted += SDL_PrivateMouseMotion(0, 0, Private->EPOC_EmulMouseX, Private->EPOC_EmulMouseY); /* Absolute position on screen */
	}

#endif
	TCallBack callback (StopSchedulerTimeOut);
	CAsyncCallBack* onecallback = new CAsyncCallBack (callback,CActive::EPriorityLow); //Service all the others first and then this
	onecallback->CallBack();
	CActiveScheduler::Start();// Run scheduler so the pointerevents and keyboard events get placed
	delete onecallback;
	onecallback = NULL;

}




void EPOC_InitOSKeymap(_THIS)
{
	int i;

	/* Initialize the key translation table */
	for ( i=0; i<SDL_TABLESIZE(keymap); ++i )
		keymap[i] = SDLK_UNKNOWN;


	/* Numbers */
	for ( i = 0; i<32; ++i ){
		keymap[' ' + i] = (SDLKey)(SDLK_SPACE+i);
	}
	/* e.g. Alphabet keys */
	for ( i = 0; i<32; ++i ){
		keymap['A' + i] = (SDLKey)(SDLK_a+i);
	}

	keymap[EStdKeyBackspace]    = SDLK_BACKSPACE;
	keymap[EStdKeyTab]          = SDLK_TAB;
	keymap[EStdKeyEnter]        = SDLK_RETURN;
	keymap[EStdKeyEscape]       = SDLK_ESCAPE;
   	keymap[EStdKeySpace]        = SDLK_SPACE;
   	keymap[EStdKeyPause]        = SDLK_PAUSE;
   	keymap[EStdKeyHome]         = SDLK_HOME;
   	keymap[EStdKeyEnd]          = SDLK_END;
   	keymap[EStdKeyPageUp]       = SDLK_PAGEUP;
   	keymap[EStdKeyPageDown]     = SDLK_PAGEDOWN;
	keymap[EStdKeyDelete]       = SDLK_DELETE;
	keymap[EStdKeyUpArrow]      = SDLK_UP;
	keymap[EStdKeyDownArrow]    = SDLK_DOWN;
	keymap[EStdKeyLeftArrow]    = SDLK_LEFT;
	keymap[EStdKeyRightArrow]   = SDLK_RIGHT;
	keymap[EStdKeyCapsLock]     = SDLK_CAPSLOCK;
	keymap[EStdKeyLeftShift]    = SDLK_LSHIFT;
	keymap[EStdKeyRightShift]   = SDLK_RSHIFT;
	keymap[EStdKeyLeftAlt]      = SDLK_LALT;
	keymap[EStdKeyRightAlt]     = SDLK_RALT;
	keymap[EStdKeyLeftCtrl]     = SDLK_LCTRL;
	keymap[EStdKeyRightCtrl]    = SDLK_RCTRL;
	keymap[EStdKeyLeftFunc]     = SDLK_LMETA;
	keymap[EStdKeyRightFunc]    = SDLK_RMETA;
	keymap[EStdKeyInsert]       = SDLK_INSERT;
	keymap[EStdKeyComma]        = SDLK_COMMA;
	keymap[EStdKeyFullStop]     = SDLK_PERIOD;
	keymap[EStdKeyForwardSlash] = SDLK_SLASH;
	keymap[EStdKeyBackSlash]    = SDLK_BACKSLASH;
	keymap[EStdKeySemiColon]    = SDLK_SEMICOLON;
	keymap[EStdKeySingleQuote]  = SDLK_QUOTE;
	keymap[EStdKeyHash]         = SDLK_HASH;
	keymap[EStdKeySquareBracketLeft]    = SDLK_LEFTBRACKET;
	keymap[EStdKeySquareBracketRight]   = SDLK_RIGHTBRACKET;
	keymap[EStdKeyMinus]        = SDLK_MINUS;
	keymap[EStdKeyEquals]       = SDLK_EQUALS;

   	keymap[EStdKeyF1]          = SDLK_F1;  /* chr + q */
   	keymap[EStdKeyF2]          = SDLK_F2;  /* chr + w */
   	keymap[EStdKeyF3]          = SDLK_F3;  /* chr + e */
   	keymap[EStdKeyF4]          = SDLK_F4;  /* chr + r */
   	keymap[EStdKeyF5]          = SDLK_F5;  /* chr + t */
   	keymap[EStdKeyF6]          = SDLK_F6;  /* chr + y */
   	keymap[EStdKeyF7]          = SDLK_F7;  /* chr + i */
   	keymap[EStdKeyF8]          = SDLK_F8;  /* chr + o */

   	keymap[EStdKeyF9]          = SDLK_F9;  /* chr + a */
   	keymap[EStdKeyF10]         = SDLK_F10; /* chr + s */
   	keymap[EStdKeyF11]         = SDLK_F11; /* chr + d */
   	keymap[EStdKeyF12]         = SDLK_F12; /* chr + f */

#ifndef SYMBIAN_SERIES_60
//	keymap[EStdQuartzKeyConfirm]	 = SDLK_RETURN;
	keymap[EStdQuartzKeyConfirm]	 = SDLK_F5;
	keymap[EStdQuartzKeyTwoWayDown] = SDLK_ESCAPE;
	keymap[EStdQuartzKeyTwoWayUp] = SDLK_SPACE;
	keymap[EStdQuartzKeyFourWayUp] = SDLK_UP;
	keymap[EStdQuartzKeyFourWayDown] = SDLK_DOWN;
	keymap[EStdQuartzKeyFourWayLeft] = SDLK_LEFT;
	keymap[EStdQuartzKeyFourWayRight] = SDLK_RIGHT;
#endif
    /* !!TODO
	EStdKeyNumLock=0x1b,
	EStdKeyScrollLock=0x1c,

	EStdKeyNkpForwardSlash=0x84,
	EStdKeyNkpAsterisk=0x85,
	EStdKeyNkpMinus=0x86,
	EStdKeyNkpPlus=0x87,
	EStdKeyNkpEnter=0x88,
	EStdKeyNkp1=0x89,
	EStdKeyNkp2=0x8a,
	EStdKeyNkp3=0x8b,
	EStdKeyNkp4=0x8c,
	EStdKeyNkp5=0x8d,
	EStdKeyNkp6=0x8e,
	EStdKeyNkp7=0x8f,
	EStdKeyNkp8=0x90,
	EStdKeyNkp9=0x91,
	EStdKeyNkp0=0x92,
	EStdKeyNkpFullStop=0x93,
    EStdKeyMenu=0x94,
    EStdKeyBacklightOn=0x95,
    EStdKeyBacklightOff=0x96,
    EStdKeyBacklightToggle=0x97,
    EStdKeyIncContrast=0x98,
    EStdKeyDecContrast=0x99,
    EStdKeySliderDown=0x9a,
    EStdKeySliderUp=0x9b,
    EStdKeyDictaphonePlay=0x9c,
    EStdKeyDictaphoneStop=0x9d,
    EStdKeyDictaphoneRecord=0x9e,
    EStdKeyHelp=0x9f,
    EStdKeyOff=0xa0,
    EStdKeyDial=0xa1,
    EStdKeyIncVolume=0xa2,
    EStdKeyDecVolume=0xa3,
    EStdKeyDevice0=0xa4,
    EStdKeyDevice1=0xa5,
    EStdKeyDevice2=0xa6,
    EStdKeyDevice3=0xa7,
    EStdKeyDevice4=0xa8,
    EStdKeyDevice5=0xa9,
    EStdKeyDevice6=0xaa,
    EStdKeyDevice7=0xab,
    EStdKeyDevice8=0xac,
    EStdKeyDevice9=0xad,
    EStdKeyDeviceA=0xae,
    EStdKeyDeviceB=0xaf,
    EStdKeyDeviceC=0xb0,
    EStdKeyDeviceD=0xb1,
    EStdKeyDeviceE=0xb2,
    EStdKeyDeviceF=0xb3,
    EStdKeyApplication0=0xb4,
    EStdKeyApplication1=0xb5,
    EStdKeyApplication2=0xb6,
    EStdKeyApplication3=0xb7,
    EStdKeyApplication4=0xb8,
    EStdKeyApplication5=0xb9,
    EStdKeyApplication6=0xba,
    EStdKeyApplication7=0xbb,
    EStdKeyApplication8=0xbc,
    EStdKeyApplication9=0xbd,
    EStdKeyApplicationA=0xbe,
    EStdKeyApplicationB=0xbf,
    EStdKeyApplicationC=0xc0,
    EStdKeyApplicationD=0xc1,
    EStdKeyApplicationE=0xc2,
    EStdKeyApplicationF=0xc3,
    EStdKeyYes=0xc4,
    EStdKeyNo=0xc5,
    EStdKeyIncBrightness=0xc6,
    EStdKeyDecBrightness=0xc7, 
    EStdKeyCaseOpen=0xc8,
    EStdKeyCaseClose=0xc9
    */

}



static SDL_keysym *TranslateKey(int scancode, SDL_keysym *keysym)
{
    char debug[256];
    //SDL_TRACE1("SDL: TranslateKey, scancode=%d", scancode); //!!

	/* Set the keysym information */ 

	keysym->scancode = scancode;

    if ((scancode >= MAX_SCANCODE) && 
        ((scancode - ENonCharacterKeyBase + 0x0081) >= MAX_SCANCODE)) {
        SDL_SetError("Too big scancode");
        keysym->scancode = SDLK_UNKNOWN;
	    keysym->mod = KMOD_NONE; 
        return keysym;
    }

	keysym->mod = SDL_GetModState(); //!!Is this right??

    /* Handle function keys: F1, F2, F3 ... */
    if (keysym->mod & KMOD_META) {
        if (scancode >= 'A' && scancode < ('A' + 24)) { /* first 32 alphapet keys */
            switch(scancode) {
                case 'Q': scancode = EStdKeyF1; break;
                case 'W': scancode = EStdKeyF2; break;
                case 'E': scancode = EStdKeyF3; break;
                case 'R': scancode = EStdKeyF4; break;
                case 'T': scancode = EStdKeyF5; break;
                case 'Y': scancode = EStdKeyF6; break;
                case 'U': scancode = EStdKeyF7; break;
                case 'I': scancode = EStdKeyF8; break;
                case 'A': scancode = EStdKeyF9; break;
                case 'S': scancode = EStdKeyF10; break;
                case 'D': scancode = EStdKeyF11; break;
                case 'F': scancode = EStdKeyF12; break;
            }
            keysym->sym = keymap[scancode];
        }
    }

    if (scancode >= ENonCharacterKeyBase) {
        // Non character keys
	    keysym->sym = keymap[scancode - 
            ENonCharacterKeyBase + 0x0081]; // !!hard coded
    } else {
		if(scancode<MAX_SCANCODE)
			keysym->sym = keymap[scancode];
		else // not mapped 
		{
			keysym->scancode = SDLK_UNKNOWN;
			keysym->mod = KMOD_NONE; 
		}
    }


	/* If UNICODE is on, get the UNICODE value for the key */
	keysym->unicode = 0;

#if 0 // !!TODO:unicode

	if ( SDL_TranslateUNICODE ) 
    {
		/* Populate the unicode field with the ASCII value */
		keysym->unicode = scancode;
	}
#endif

    //!!
    //sprintf(debug, "SDL: TranslateKey: keysym->scancode=%d, keysym->sym=%d, keysym->mod=%d",
    //    keysym->scancode, keysym->sym, keysym->mod);
    //SDL_TRACE(debug); //!!

	return(keysym);
}

}; /* extern "C" */


